<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\Node;
use Drupal\content_moderation\Entity\ContentModerationState;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a content type for books and chapters.
    $this->drupalCreateContentType(['type' => 'book', 'name' => 'Book']);
    $this->drupalCreateContentType(['type' => 'chapter', 'name' => 'Chapter']);

    // Enable moderation for book and chapter content types.
    $workflow = $this->container->get('entity_type.manager')
      ->getStorage('workflow')
      ->load('editorial');
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'book');
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'chapter');
    $workflow->save();

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer nodes',
      'view any unpublished content',
      'use editorial transition create_new_draft',
      'use editorial transition publish',
    ]);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests that moderation states are synchronized between books and chapters.
   */
  public function testModerationStateSync() {
    // Create a book.
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);

    // Create a chapter and add it to the book.
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'book' => ['bid' => $book->id()],
      'moderation_state' => 'draft',
    ]);

    // Publish the book.
    $book->set('moderation_state', 'published');
    $book->save();

    // Reload the chapter and check its moderation state.
    $chapter = Node::load($chapter->id());
    $this->assertEquals('published', $chapter->get('moderation_state')->value, 'Chapter moderation state is synchronized with the book.');
  }

}